package pps.core.common.utils;

import cn.hutool.core.date.DateUtil;
import xstartup.base.XContext;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

public enum FiveSecondsTimerJobHelper implements Runnable {
    INSTANCE;

    private ReentrantLock lock = new ReentrantLock();
    private Thread timerThread;
    private List<Runnable> tasks = new ArrayList<>();
    private XContext context;

    public XContext getContext() {
        return context;
    }

    public void setContext(XContext context) {
        this.context = context;
    }

    public void register(Runnable runnable) {
        lock.lock();
        try {
            this.tasks.add(runnable);
            CounterBuilder.globalCounterBuilder.setFieldValue("FiveSecondsTimer-Tasks", this.tasks.size() + "");
            if (timerThread == null) {
                // 启动线程
                timerThread = new Thread(this);
                timerThread.setName("FiveSecondsTimerJobThread");
                timerThread.start();
            }
        } finally {
            lock.unlock();
        }
    }


    @Override
    public void run() {
        try {
            while (true) {
                this.tasks.forEach(task -> {
                    try {
                        task.run();
                    } catch (Exception ex) {
                        CounterBuilder.globalCounterBuilder.addCounter("FiveSecondsTimerTaskException");
                        CounterBuilder.globalCounterBuilder.addCounter(ex, this.getClass());
                    }
                });
                Thread.sleep(5000);
                CounterBuilder.globalCounterBuilder.addCounter("FiveSecondsTimer");
                CounterBuilder.globalCounterBuilder.setFieldValue("FiveSecondsTimer-LastTime", DateUtil.date().toString());
            }
        } catch (Exception ex) {
            CounterBuilder.globalCounterBuilder.addCounter(ex, this.getClass());
        }
    }
}
